@@ -1,19 +1,19 @@
 """DVC re-implementation of fsspec's dedicated async event loop."""
+
 import asyncio
 import os
 import threading
-from typing import Any, List, Optional
+from typing import Any, Optional
 
 from fsspec.asyn import (  # noqa: F401, pylint:disable=unused-import
-    _selector_policy,
     sync,
     sync_wrapper,
 )
 
 # dedicated async IO thread
-iothread: List[Optional[threading.Thread]] = [None]
+iothread: list[Optional[threading.Thread]] = [None]
 # global DVC event loop
-default_loop: List[Optional[asyncio.AbstractEventLoop]] = [None]
+default_loop: list[Optional[asyncio.AbstractEventLoop]] = [None]
 lock = threading.Lock()
 
 
@@ -22,8 +22,7 @@ def get_loop() -> asyncio.AbstractEventLoop:
     if default_loop[0] is None:
         with lock:
             if default_loop[0] is None:
-                with _selector_policy():
-                    default_loop[0] = asyncio.new_event_loop()
+                default_loop[0] = asyncio.new_event_loop()
                 loop = default_loop[0]
                 th = threading.Thread(
                     target=loop.run_forever,  # type: ignore[attr-defined]
