@@ -2,7 +2,9 @@ import fnmatch
 import locale
 import logging
 import os
+import re
 import stat
+from collections.abc import Iterable, Iterator, Mapping
 from contextlib import closing
 from functools import partial
 from io import BytesIO, StringIO
@@ -10,45 +12,50 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
-    Dict,
-    Iterable,
-    Iterator,
-    List,
-    Mapping,
     Optional,
-    Tuple,
     Union,
 )
 
+from dulwich.config import ConfigFile, StackedConfig
+from dulwich.walk import ORDER_DATE
 from funcy import cached_property, reraise
 
 from scmrepo.exceptions import AuthError, CloneError, InvalidRemote, RevError, SCMError
+from scmrepo.git.backend.base import BaseGitBackend, SyncStatus
+from scmrepo.git.config import Config
+from scmrepo.git.objects import GitObject, GitTag
 from scmrepo.progress import GitProgressReporter
 from scmrepo.utils import relpath
 
-from ...objects import GitObject
-from ..base import BaseGitBackend, SyncStatus
-
 if TYPE_CHECKING:
     from dulwich.client import SSHVendor
+    from dulwich.config import ConfigFile
     from dulwich.repo import Repo
 
+    from scmrepo.git.objects import GitCommit
     from scmrepo.progress import GitProgressEvent
 
-    from ...objects import GitCommit
-
 
 logger = logging.getLogger(__name__)
 
 
 class DulwichObject(GitObject):
-    def __init__(self, repo, name, mode, sha):
+    def __init__(self, repo, name, mode, sha) -> None:
         self.repo = repo
         self._name = name
         self._mode = mode
         self._sha = sha
 
-    def open(self, mode: str = "r", encoding: str = None):
+    def open(  # pylint: disable=unused-argument
+        self,
+        mode: str = "r",
+        encoding: Optional[str] = None,
+        raw: bool = True,
+        rev: Optional[str] = None,
+        **kwargs,
+    ):
+        if not raw:
+            raise NotImplementedError
         if not encoding:
             encoding = locale.getpreferredencoding(False)
         # NOTE: we didn't load the object before as Dulwich will also try to
@@ -70,7 +77,7 @@ class DulwichObject(GitObject):
 
     def scandir(self) -> Iterable["DulwichObject"]:
         tree = self.repo[self._sha]
-        for entry in tree.iteritems():  # noqa: B301
+        for entry in tree.iteritems():
             yield DulwichObject(self.repo, entry.path.decode(), entry.mode, entry.sha)
 
     @cached_property
@@ -99,6 +106,7 @@ class DulwichProgressReporter(GitProgressReporter):
 
 
 def _get_ssh_vendor() -> "SSHVendor":
+    import shutil
     import sys
 
     from dulwich.client import SubprocessSSHVendor
@@ -110,7 +118,7 @@ def _get_ssh_vendor() -> "SSHVendor":
         logger.debug("dulwich: Using environment GIT_SSH_COMMAND '%s'", ssh_command)
         return SubprocessSSHVendor()
 
-    if sys.platform == "win32" and os.environ.get("MSYSTEM"):
+    if sys.platform == "win32" and os.environ.get("MSYSTEM") and shutil.which("ssh"):
         # see https://github.com/iterative/dvc/issues/7702
         logger.debug(
             "dulwich: native win32 Python inside MSYS2/git-bash, using MSYS2 OpenSSH"
@@ -119,7 +127,7 @@ def _get_ssh_vendor() -> "SSHVendor":
 
     default_config = os.path.expanduser(os.path.join("~", ".ssh", "config"))
     unsupported = list(get_unsupported_opts([default_config]))
-    if unsupported:
+    if unsupported and shutil.which("ssh"):
         logger.debug(
             "dulwich: unsupported SSH config option(s) '%s', using system OpenSSH",
             ", ".join(unsupported),
@@ -128,6 +136,35 @@ def _get_ssh_vendor() -> "SSHVendor":
     return AsyncSSHVendor()
 
 
+class DulwichConfig(Config):
+    def __init__(self, config: Union["ConfigFile", "StackedConfig"]) -> None:
+        self._config = config
+
+    @property
+    def encoding(self) -> str:
+        from dulwich.config import ConfigFile
+
+        if isinstance(self._config, ConfigFile):
+            return self._config.encoding
+        return self._config.backends[0].encoding
+
+    def get(self, section: tuple[str, ...], name: str) -> str:
+        """Return the specified setting as a string."""
+        return self._config.get(section, name).decode(self.encoding)
+
+    def get_bool(self, section: tuple[str, ...], name: str) -> bool:
+        """Return the specified setting as a boolean."""
+        value = self._config.get_boolean(section, name)
+        if value is None:
+            raise ValueError("setting is not a valid boolean")
+        return value
+
+    def get_multivar(self, section: tuple[str, ...], name: str) -> Iterator[str]:
+        """Iterate over string values in the specified multivar setting."""
+        for value in self._config.get_multivar(section, name):
+            yield value.decode(self.encoding)
+
+
 class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
     """Dulwich Git backend."""
 
@@ -147,7 +184,7 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
 
     def __init__(  # pylint:disable=W0231
         self, root_dir=os.curdir, search_parent_directories=True
-    ):
+    ) -> None:
         from dulwich.errors import NotGitRepository
         from dulwich.repo import Repo
 
@@ -159,17 +196,22 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
         except NotGitRepository as exc:
             raise SCMError(f"{root_dir} is not a git repository") from exc
 
-        self._submodules: Dict[str, str] = self._find_submodules()
+        self._submodules: dict[str, str] = self._find_submodules()
         self._stashes: dict = {}
 
-    def _find_submodules(self) -> Dict[str, str]:
+    def list_submodules(self) -> list[str]:
+        raise NotImplementedError
+
+    def _find_submodules(self) -> dict[str, str]:
         """Return dict mapping submodule names to submodule paths.
 
         Submodule paths will be relative to Git repo root.
         """
+
         from dulwich.config import ConfigFile, parse_submodules
 
-        submodules: Dict[str, str] = {}
+        submodules: dict[str, str] = {}
+        assert self.root_dir
         config_path = os.path.join(self.root_dir, ".gitmodules")
         if os.path.isfile(config_path):
             config = ConfigFile.from_path(config_path)
@@ -181,22 +223,26 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
         self.repo.close()
 
     @property
-    def root_dir(self) -> str:
+    def root_dir(self) -> Optional[str]:
         return self.repo.path
 
     @classmethod
     def clone(
         cls,
         url: str,
-        to_path: str,
+        to_path: Union[str, os.PathLike[str]],
         shallow_branch: Optional[str] = None,
-        progress: Callable[["GitProgressEvent"], None] = None,
+        progress: Optional[Callable[["GitProgressEvent"], None]] = None,
+        bare: bool = False,
+        mirror: bool = False,
     ):
         from urllib.parse import urlparse
 
         from dulwich.porcelain import NoneStream
         from dulwich.porcelain import clone as git_clone
 
+        if mirror:
+            bare = True
         parsed = urlparse(url)
         try:
             clone_from = partial(
@@ -204,8 +250,9 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
                 url,
                 target=to_path,
                 errstream=(
-                    DulwichProgressReporter(progress) if progress else NoneStream()
+                    DulwichProgressReporter(progress) if progress else NoneStream()  # type: ignore[arg-type]
                 ),
+                bare=bare,
             )
             if shallow_branch:
                 # NOTE: dulwich only supports shallow/depth for non-local
@@ -220,16 +267,22 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
                 repo = clone_from()
 
             with closing(repo):
-                cls._set_default_tracking_branch(repo)
+                if mirror:
+                    cls._set_mirror(repo, progress=progress)
+                else:
+                    cls._set_default_tracking_branch(repo)
         except Exception as exc:
-            raise CloneError(url, to_path) from exc
+            raise CloneError(url, os.fsdecode(to_path)) from exc
 
     @staticmethod
     def _set_default_tracking_branch(repo: "Repo"):
         from dulwich.refs import LOCAL_BRANCH_PREFIX, parse_symref_value
 
+        head_ref = repo.refs.read_ref(b"HEAD")
+        if head_ref is None:
+            return
         try:
-            ref = parse_symref_value(repo.refs.read_ref(b"HEAD"))
+            ref = parse_symref_value(head_ref)
         except ValueError:
             return
         if ref.startswith(LOCAL_BRANCH_PREFIX):
@@ -239,6 +292,27 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
             config.set(section, b"remote", b"origin")
             config.set(section, b"merge", ref)
 
+    @staticmethod
+    def _set_mirror(
+        repo: "Repo", progress: Optional[Callable[["GitProgressEvent"], None]] = None
+    ):
+        from dulwich.porcelain import NoneStream, fetch
+
+        config = repo.get_config()
+        section = config[(b"remote", b"origin")]
+        try:
+            del section[b"fetch"]
+        except KeyError:
+            pass
+        section[b"fetch"] = b"+refs/*:refs/*"
+        section[b"mirror"] = b"true"
+        config.write_to_path()
+        fetch(
+            repo,
+            remote_location=b"origin",
+            errstream=(DulwichProgressReporter(progress) if progress else NoneStream()),  # type: ignore[arg-type]
+        )
+
     @staticmethod
     def init(path: str, bare: bool = False) -> None:
         from dulwich.porcelain import init
@@ -259,11 +333,13 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
 
         paths = [paths] if isinstance(paths, str) else list(paths)
 
+        worktree = self.repo.get_worktree()
+
         if update and not paths:
-            self.repo.stage(list(self.repo.open_index()))
+            worktree.stage(list(self.repo.open_index()))
             return
 
-        files: List[bytes] = [
+        files: list[bytes] = [
             os.fsencode(fpath) for fpath in self._expand_paths(paths, force=force)
         ]
         if update:
@@ -271,30 +347,16 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
             if os.name == "nt":
                 # NOTE: we need git/unix separator to compare against index
                 # paths but repo.stage() expects to be called with OS paths
-                self.repo.stage(
+                worktree.stage(
                     [fname for fname in files if fname.replace(b"\\", b"/") in index]
                 )
             else:
-                self.repo.stage([fname for fname in files if fname in index])
+                worktree.stage([fname for fname in files if fname in index])
         else:
-            self.repo.stage(files)
+            worktree.stage(files)
 
-    def _expand_paths(self, paths: List[str], force: bool = False) -> Iterator[str]:
+    def _expand_paths(self, paths: list[str], force: bool = False) -> Iterator[str]:
         for path in paths:
-            if not os.path.isabs(path) and self._submodules:
-                # NOTE: If path is inside a submodule, Dulwich expects the
-                # staged paths to be relative to the submodule root (not the
-                # parent git repo root). We append path to root_dir here so
-                # that the result of relpath(path, root_dir) is actually the
-                # path relative to the submodule root.
-                fs_path = relpath(path, self.root_dir)
-                for sm_path in self._submodules.values():
-                    if fs_path.startswith(sm_path):
-                        path = os.path.join(
-                            self.root_dir,
-                            relpath(fs_path, sm_path),
-                        )
-                        break
             if os.path.isdir(path):
                 for root, _, fs in os.walk(path):
                     for fpath in fs:
@@ -313,7 +375,7 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
 
         with reraise((Error, CommitError), SCMError("Git commit failed")):
             try:
-                commit(self.root_dir, message=msg, no_verify=no_verify)
+                commit(self.repo, message=msg, no_verify=no_verify)
             except InvalidUserIdentity as exc:
                 raise SCMError("Git username and email must be configured") from exc
             except TimezoneFormatError as exc:
@@ -356,12 +418,29 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
         from dulwich.porcelain import Error, branch_create
 
         try:
-            branch_create(self.root_dir, branch)
+            branch_create(self.repo, branch)
         except Error as exc:
             raise SCMError(f"Failed to create branch '{branch}'") from exc
 
-    def tag(self, tag: str):
-        raise NotImplementedError
+    def tag(
+        self,
+        tag: str,
+        target: Optional[str] = None,
+        annotated: bool = False,
+        message: Optional[str] = None,
+    ):
+        from dulwich.porcelain import Error, tag_create
+
+        if annotated and not message:
+            raise SCMError("message is required for annotated tag")
+        with reraise(Error, SCMError("Failed to create tag")):
+            tag_create(
+                self.repo,
+                os.fsencode(tag),
+                objectish=target or "HEAD",
+                annotated=annotated,
+                message=message.encode("utf-8") if message else None,
+            )
 
     def untracked_files(self) -> Iterable[str]:
         _staged, _unstaged, untracked = self.status()
@@ -370,26 +449,46 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
     def is_tracked(self, path: str) -> bool:
         rel = relpath(path, self.root_dir).replace(os.path.sep, "/").encode()
         rel_dir = rel + b"/"
-        for p in self.repo.open_index():
-            if p == rel or p.startswith(rel_dir):
-                return True
-        return False
+        return any(p == rel or p.startswith(rel_dir) for p in self.repo.open_index())
 
     def is_dirty(self, untracked_files: bool = False) -> bool:
-        kwargs: Dict[str, Any] = {} if untracked_files else {"untracked_files": "no"}
+        kwargs: dict[str, Any] = {} if untracked_files else {"untracked_files": "no"}
         return any(self.status(**kwargs))
 
     def active_branch(self) -> str:
         raise NotImplementedError
 
-    def list_branches(self) -> Iterable[str]:
+    def active_branch_remote(self) -> str:
         raise NotImplementedError
 
+    def list_branches(self) -> Iterable[str]:
+        base = "refs/heads/"
+        return sorted(ref[len(base) :] for ref in self.iter_refs(base))
+
     def list_tags(self) -> Iterable[str]:
-        raise NotImplementedError
+        base = "refs/tags/"
+        return sorted(ref[len(base) :] for ref in self.iter_refs(base))
 
     def list_all_commits(self) -> Iterable[str]:
-        raise NotImplementedError
+        from dulwich.objects import Tag
+
+        repo = self.repo
+        starting_points: list[bytes] = []
+
+        # HEAD
+        head_rev = self.get_ref("HEAD")
+        if head_rev:
+            starting_points.append(head_rev.encode("utf-8"))
+
+        # Branches and remotes
+        for ref in repo.refs:
+            if ref.startswith((b"refs/heads/", b"refs/remotes/", b"refs/tags/")):
+                if isinstance(repo.refs[ref], Tag):
+                    ref = self.repo.get_peeled(repo.refs[ref])
+                starting_points.append(repo.refs[ref])
+
+        walker = self.repo.get_walker(include=starting_points, order=ORDER_DATE)
+        return [e.commit.id.decode() for e in walker]
 
     def get_tree_obj(self, rev: str, **kwargs) -> DulwichObject:
         from dulwich.objectspec import parse_tree
@@ -496,19 +595,19 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
 
         try:
             _remote, location = get_remote_repo(self.repo, url)
-            client, path = get_transport_and_path(location, **kwargs)
+            _config = kwargs.pop("config", StackedConfig.default())
+            client, path = get_transport_and_path(location, config=_config, **kwargs)
         except Exception as exc:
             raise InvalidRemote(url) from exc
 
         try:
+            refs = client.get_refs(path.encode()).refs
+
             if base:
-                yield from (
-                    os.fsdecode(ref)
-                    for ref in client.get_refs(path)
-                    if ref.startswith(os.fsencode(base))
-                )
+                base_b = os.fsencode(base)
+                yield from (os.fsdecode(ref) for ref in refs if ref.startswith(base_b))
             else:
-                yield from (os.fsdecode(ref) for ref in client.get_refs(path))
+                yield from (os.fsdecode(ref) for ref in refs)
         except NotGitRepository as exc:
             raise InvalidRemote(url) from exc
         except HTTPUnauthorized as exc:
@@ -523,7 +622,7 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
         refspecs: Union[str, Iterable[str]],
         force: bool = False,
         on_diverged: Optional[Callable[[str, str], bool]] = None,
-        progress: Callable[["GitProgressEvent"], None] = None,
+        progress: Optional[Callable[["GitProgressEvent"], None]] = None,
         **kwargs,
     ) -> Mapping[str, SyncStatus]:
         from dulwich.client import HTTPUnauthorized, get_transport_and_path
@@ -533,21 +632,28 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
 
         try:
             _remote, location = get_remote_repo(self.repo, url)
-            client, path = get_transport_and_path(location, **kwargs)
+            _config = kwargs.pop("config", StackedConfig.default())
+            client, path = get_transport_and_path(location, config=_config, **kwargs)
         except Exception as exc:
             raise SCMError(f"'{url}' is not a valid Git remote or URL") from exc
 
         change_result = {}
-        selected_refs = []
+        selected_refs: list[tuple[Optional[bytes], Optional[bytes], bool]] = []
 
         def update_refs(refs):
             from dulwich.objects import ZERO_SHA
 
+            _refspecs = (
+                os.fsencode(refspecs)
+                if isinstance(refspecs, str)
+                else [os.fsencode(refspec) for refspec in refspecs]
+            )
             selected_refs.extend(
-                parse_reftuples(self.repo.refs, refs, refspecs, force=force)
+                parse_reftuples(self.repo.refs, refs, _refspecs, force=force)
             )
             new_refs = {}
             for lh, rh, _ in selected_refs:
+                assert rh is not None
                 refname = os.fsdecode(rh)
                 if rh in refs and lh is not None:
                     if refs[rh] == self.repo.refs[lh]:
@@ -578,9 +684,9 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
 
         try:
             result = client.send_pack(
-                path,
+                path.encode(),
                 update_refs,
-                generate_pack_data=self.repo.generate_pack_data,
+                generate_pack_data=self.repo.generate_pack_data,  # type: ignore[arg-type]
                 progress=(DulwichProgressReporter(progress) if progress else None),
             )
         except (NotGitRepository, SendPackError) as exc:
@@ -607,7 +713,7 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
         refspecs: Union[str, Iterable[str]],
         force: bool = False,
         on_diverged: Optional[Callable[[str, str], bool]] = None,
-        progress: Callable[["GitProgressEvent"], None] = None,
+        progress: Optional[Callable[["GitProgressEvent"], None]] = None,
         **kwargs,
     ) -> Mapping[str, SyncStatus]:
         from dulwich.client import get_transport_and_path
@@ -616,12 +722,12 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
         from dulwich.porcelain import DivergedBranches, check_diverged, get_remote_repo
         from dulwich.refs import DictRefsContainer
 
-        fetch_refs = []
+        fetch_refs: list[tuple[Optional[bytes], Optional[bytes], bool]] = []
 
         def determine_wants(
-            remote_refs: Dict[bytes, bytes],
+            remote_refs: dict[bytes, bytes],
             depth: Optional[int] = None,  # pylint: disable=unused-argument
-        ) -> List[bytes]:
+        ) -> list[bytes]:
             fetch_refs.extend(
                 parse_reftuples(
                     DictRefsContainer(remote_refs),
@@ -635,45 +741,47 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
             return [
                 remote_refs[lh]
                 for (lh, _, _) in fetch_refs
-                if remote_refs[lh] not in self.repo.object_store
+                if lh is not None and remote_refs[lh] not in self.repo.object_store
             ]
 
         with reraise(Exception, SCMError(f"'{url}' is not a valid Git remote or URL")):
             _remote, location = get_remote_repo(self.repo, url)
-            client, path = get_transport_and_path(location, **kwargs)
+            _config = kwargs.pop("config", StackedConfig.default())
+            client, path = get_transport_and_path(location, config=_config, **kwargs)
 
         with reraise(
             (NotGitRepository, KeyError),
             SCMError(f"Git failed to fetch ref from '{url}'"),
         ):
             fetch_result = client.fetch(
-                path,
+                path.encode(),
                 self.repo,
                 progress=DulwichProgressReporter(progress) if progress else None,
-                determine_wants=determine_wants,
+                determine_wants=determine_wants,  # type: ignore[arg-type]
             )
 
             result = {}
 
             for lh, rh, _ in fetch_refs:
+                assert rh is not None
                 refname = os.fsdecode(rh)
+                assert lh is not None
+                fetch_ref_lh = fetch_result.refs[lh]
+                assert fetch_ref_lh is not None
                 if rh in self.repo.refs:
-                    if self.repo.refs[rh] == fetch_result.refs[lh]:
+                    if self.repo.refs[rh] == fetch_ref_lh:
                         result[refname] = SyncStatus.UP_TO_DATE
                         continue
                     try:
                         check_diverged(
                             self.repo,
                             self.repo.refs[rh],
-                            fetch_result.refs[lh],
+                            fetch_ref_lh,
                         )
                     except DivergedBranches:
                         if not force:
                             overwrite = (
-                                on_diverged(
-                                    os.fsdecode(rh),
-                                    os.fsdecode(fetch_result.refs[lh]),
-                                )
+                                on_diverged(os.fsdecode(rh), os.fsdecode(fetch_ref_lh))
                                 if on_diverged
                                 else False
                             )
@@ -681,7 +789,7 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
                                 result[refname] = SyncStatus.DIVERGED
                                 continue
 
-                self.repo.refs[rh] = fetch_result.refs[lh]
+                self.repo.refs[rh] = fetch_ref_lh
                 result[refname] = SyncStatus.SUCCESS
         return result
 
@@ -694,7 +802,7 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
         ref: str,
         message: Optional[str] = None,
         include_untracked: bool = False,
-    ) -> Tuple[Optional[str], bool]:
+    ) -> tuple[Optional[str], bool]:
         from dulwich.repo import InvalidUserIdentity
 
         from scmrepo.git import Stash
@@ -748,8 +856,8 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
     ) -> Mapping[str, Optional[str]]:
         if not base:
             base = "refs/tags"
-        rev_mapping: Dict[str, Optional[str]] = {}
-        results: Dict[str, Optional[str]] = {}
+        rev_mapping: dict[str, Optional[str]] = {}
+        results: dict[str, Optional[str]] = {}
         for ref in self.iter_refs(base=base):
             if (match and not fnmatch.fnmatch(ref, match)) or (
                 exclude and fnmatch.fnmatch(ref, exclude)
@@ -759,10 +867,11 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
             if revision and revision not in rev_mapping:
                 rev_mapping[revision] = ref
         for rev in revs:
-            results[rev] = rev_mapping.get(rev, None)
+            results[rev] = rev_mapping.get(rev)
         return results
 
     def diff(self, rev_a: str, rev_b: str, binary=False) -> str:
+        from dulwich.objects import Commit
         from dulwich.patch import write_tree_diff
 
         try:
@@ -771,11 +880,14 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
         except KeyError as exc:
             raise RevError("Invalid revision") from exc
 
+        assert isinstance(commit_a, Commit)
+        assert isinstance(commit_b, Commit)
+
         buf = BytesIO()
         write_tree_diff(buf, self.repo.object_store, commit_a.tree, commit_b.tree)
         return buf.getvalue().decode("utf-8")
 
-    def reset(self, hard: bool = False, paths: Iterable[str] = None):
+    def reset(self, hard: bool = False, paths: Optional[Iterable[str]] = None):
         raise NotImplementedError
 
     def checkout_index(
@@ -789,13 +901,13 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
 
     def status(
         self, ignored: bool = False, untracked_files: str = "all"
-    ) -> Tuple[Mapping[str, Iterable[str]], Iterable[str], Iterable[str]]:
+    ) -> tuple[Mapping[str, Iterable[str]], Iterable[str], Iterable[str]]:
         from dulwich.porcelain import Error
         from dulwich.porcelain import status as git_status
 
         with reraise(Error, SCMError("Git status failed")):
             staged, unstaged, untracked = git_status(
-                self.root_dir, ignored=ignored, untracked_files=untracked_files
+                self.repo, ignored=ignored, untracked_files=untracked_files
             )
 
         return (
@@ -826,6 +938,7 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
 
         try:
             _, location = get_remote_repo(self.repo, url)
+            _config = kwargs.pop("config", StackedConfig.default())
             client, path = get_transport_and_path(location, **kwargs)
         except Exception as exc:
             raise InvalidRemote(url) from exc
@@ -834,7 +947,82 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
         ):
             raise InvalidRemote(url)
 
+    def get_remote_url(self, remote: str) -> str:
+        from dulwich.porcelain import get_remote_repo
+
+        remote_name, location = get_remote_repo(self.repo, remote)
+        if not remote_name:
+            raise InvalidRemote(remote)
+        return location
+
     def check_ref_format(self, refname: str) -> bool:
         from dulwich.refs import check_ref_format
 
         return check_ref_format(refname.encode())
+
+    def get_tag(self, name: str) -> Optional[Union[str, "GitTag"]]:
+        from dulwich.objects import Tag
+
+        name_b = os.fsencode(f"refs/tags/{name}")
+        try:
+            ref = self.repo.refs[name_b]
+        except KeyError:
+            return None
+        if ref in self.repo:
+            shafile = self.repo[ref]
+            if isinstance(shafile, Tag):
+                tag = shafile
+                _typ, target_sha = tag.object
+                tagger_name, tagger_email = _parse_identity(tag.tagger.decode("utf-8"))
+                return GitTag(
+                    os.fsdecode(tag.name),
+                    tag.id.decode("ascii"),
+                    target_sha.decode("ascii"),
+                    tagger_name,
+                    tagger_email,
+                    tag.tag_time,
+                    tag.tag_timezone,
+                    tag.message.decode("utf-8"),
+                )
+        return os.fsdecode(ref)
+
+    def get_config(self, path: Optional[str] = None) -> "Config":
+        from dulwich.config import ConfigFile
+
+        if path:
+            return DulwichConfig(ConfigFile.from_path(path))
+        return DulwichConfig(self.repo.get_config_stack())
+
+    def check_attr(
+        self,
+        path: str,
+        attr: str,
+        source: Optional[str] = None,
+    ) -> Optional[Union[bool, str]]:
+        raise NotImplementedError
+
+
+_IDENTITY_RE = re.compile(r"(?P<name>.+)\s+<(?P<email>.+)>")
+
+
+def _parse_identity(identity: str) -> tuple[str, str]:
+    m = _IDENTITY_RE.match(identity)
+    if not m:
+        raise SCMError("Could not parse tagger identity '{identity}'")
+    return m.group("name"), m.group("email")
+
+
+def ls_remote(url: str) -> dict[str, Optional[str]]:
+    from dulwich import porcelain
+    from dulwich.client import HTTPUnauthorized
+
+    try:
+        refs = porcelain.ls_remote(url).refs
+        return {
+            os.fsdecode(ref): sha.decode("ascii") if sha is not None else None
+            for ref, sha in refs.items()
+        }
+    except HTTPUnauthorized as exc:
+        raise AuthError(url) from exc
+    except Exception as exc:
+        raise InvalidRemote(url) from exc
