@@ -2,7 +2,8 @@ import asyncio
 import os
 import subprocess
 import sys
-from typing import Any, AsyncIterator, Dict, Iterator
+from collections.abc import AsyncIterator, Iterator
+from typing import Any
 
 import asyncssh
 import pygit2
@@ -35,7 +36,7 @@ def pytest_collection_modifyitems(config, items):
 
 
 @pytest.fixture(autouse=True)
-def isolate(tmp_dir_factory: TempDirFactory, monkeypatch: pytest.MonkeyPatch) -> None:
+def _isolate(tmp_dir_factory: TempDirFactory, monkeypatch: pytest.MonkeyPatch) -> None:
     path = tmp_dir_factory.mktemp("mock")
     home_dir = path / "home"
     home_dir.mkdir()
@@ -47,6 +48,7 @@ def isolate(tmp_dir_factory: TempDirFactory, monkeypatch: pytest.MonkeyPatch) ->
         monkeypatch.setenv("HOMEPATH", home_path)
     else:
         monkeypatch.setenv("HOME", str(home_dir))
+        monkeypatch.setenv("XDG_CONFIG_HOME", str(home_dir / ".config"))
 
     monkeypatch.setenv("GIT_CONFIG_NOSYSTEM", "1")
     contents = b"""
@@ -57,7 +59,7 @@ email=dvctester@example.com
 defaultBranch=master
 """
     (home_dir / ".gitconfig").write_bytes(contents)
-    pygit2.settings.search_path[pygit2.GIT_CONFIG_LEVEL_GLOBAL] = str(home_dir)
+    pygit2.settings.search_path[pygit2.GIT_CONFIG_LEVEL_GLOBAL] = str(home_dir)  # type: ignore[attr-defined, index]
 
 
 @pytest.fixture
@@ -103,7 +105,7 @@ def remote_git_dir(tmp_dir_factory: TempDirFactory):
 
 @pytest.fixture(scope="session")
 def docker(request: pytest.FixtureRequest):
-    for cmd in [("docker", "ps"), ("docker-compose", "version")]:
+    for cmd in [("docker", "ps"), ("docker", "compose", "version")]:
         try:
             subprocess.check_call(
                 cmd,
@@ -117,13 +119,13 @@ def docker(request: pytest.FixtureRequest):
         pytest.skip("disabled for Windows on Github Actions")
 
     pytest.importorskip("pytest_docker")
-    yield request.getfixturevalue("docker_services")
+    return request.getfixturevalue("docker_services")
 
 
 @pytest.fixture
 def ssh_conn_info(
     docker,  # pylint: disable=unused-argument
-) -> Dict[str, Any]:
+) -> dict[str, Any]:
     conn_info = {
         "host": "127.0.0.1",
         "port": docker.port_for("git-server", 2222),
@@ -139,7 +141,7 @@ def ssh_conn_info(
                 assert result.returncode == 0
                 async with conn.start_sftp_client() as sftp:
                     assert await sftp.exists("/")
-        except Exception:  # pylint: disable=broad-except
+        except Exception:  # noqa: BLE001 # pylint: disable=broad-except
             return False
         return True
 
@@ -152,7 +154,7 @@ def ssh_conn_info(
 
 @pytest.fixture
 async def ssh_connection(
-    ssh_conn_info: Dict[str, Any],
+    ssh_conn_info: dict[str, Any],
 ) -> AsyncIterator[asyncssh.connection.SSHClientConnection]:
     async with asyncssh.connect(**ssh_conn_info) as conn:
         yield conn
