@@ -1,26 +1,42 @@
 import os
 import shutil
-from typing import Any, Dict, List, Optional, Type
+import time
+from pathlib import Path
+from typing import Any, Optional
 
 import pytest
 from asyncssh import SFTPClient
 from asyncssh.connection import SSHClientConnection
 from dulwich.client import LocalGitClient
 from git import Repo as GitPythonRepo
+from proxy import TestCase as ProxyTestCase
 from pygit2 import GitError
-from pygit2.remote import Remote  # type: ignore
+from pygit2.remotes import Remote
 from pytest_mock import MockerFixture
 from pytest_test_utils import TempDirFactory, TmpDir
 from pytest_test_utils.matchers import Matcher
 
-from scmrepo.exceptions import InvalidRemote, MergeConflictError, RevError, SCMError
-from scmrepo.git import Git
+from scmrepo.exceptions import (
+    InvalidRemote,
+    MergeConflictError,
+    RevError,
+    SCMError,
+)
+from scmrepo.git import Git, GitBackends
+from scmrepo.git.objects import GitTag
 
 from .conftest import backends
 
 # pylint: disable=redefined-outer-name,unused-argument,protected-access
 
 
+BAD_PROXY_CONFIG = """[http]
+proxy = "http://bad-proxy.dvc.org"
+[https]
+proxy = "http://bad-proxy.dvc.org"
+"""
+
+
 @pytest.fixture
 def submodule_dir(tmp_dir: TmpDir, scm: Git):
     scm.commit("init")
@@ -30,7 +46,7 @@ def submodule_dir(tmp_dir: TmpDir, scm: Git):
     subrepo.create_submodule(subrepo_path, subrepo_path, subrepo.git_dir)
     subrepo.close()
 
-    yield tmp_dir / subrepo_path
+    return tmp_dir / subrepo_path
 
 
 def test_git_init(tmp_dir: TmpDir, git_backend: str):
@@ -201,7 +217,7 @@ def test_get_ref(tmp_dir: TmpDir, scm: Git, git: Git):
             os.path.join(".git", "refs", "foo", "baz"): "ref: refs/heads/master",
         }
     )
-    scm.tag(["-a", "annotated", "-m", "Annotated Tag"])
+    scm.tag("annotated", annotated=True, message="Annotated Tag")
 
     assert init_rev == git.get_ref("refs/foo/bar")
     assert init_rev == git.get_ref("refs/foo/baz")
@@ -361,9 +377,9 @@ def test_fetch_refspecs(
     }
     assert baz_rev == scm.get_ref("refs/foo/baz")
 
+    mocker.patch.object(LocalGitClient, "fetch", side_effect=KeyError)
+    mocker.patch.object(Remote, "fetch", side_effect=GitError)
     with pytest.raises(SCMError):
-        mocker.patch.object(LocalGitClient, "fetch", side_effect=KeyError)
-        mocker.patch.object(Remote, "fetch", side_effect=GitError)
         git.fetch_refspecs(remote, "refs/foo/bar:refs/foo/bar")
 
     assert len(scm.pygit2.repo.remotes) == 1
@@ -393,9 +409,9 @@ def test_iter_remote_refs(
     with pytest.raises(InvalidRemote):
         set(git.iter_remote_refs("bad-remote"))
 
+    tmp_directory = tmp_dir_factory.mktemp("not_a_git_repo")
+    remote = f"file://{tmp_directory.as_posix()}"
     with pytest.raises(InvalidRemote):
-        tmp_directory = tmp_dir_factory.mktemp("not_a_git_repo")
-        remote = f"file://{tmp_directory.as_posix()}"
         set(git.iter_remote_refs(remote))
 
     remote = url if use_url else "origin"
@@ -407,21 +423,170 @@ def test_iter_remote_refs(
     } == set(git.iter_remote_refs(remote))
 
 
-@pytest.mark.skip_git_backend("dulwich", "pygit2")
-def test_list_all_commits(tmp_dir: TmpDir, scm: Git, git: Git, matcher: Type[Matcher]):
-    def _gen(s):
-        tmp_dir.gen(s, s)
-        scm.add_commit(s, message=s)
-        return scm.get_rev()
+def _gen(scm: Git, s: str, commit_timestamp: Optional[float] = None) -> str:
+    with open(s, mode="w") as f:
+        f.write(s)
+    scm.dulwich.add([s])
+    scm.dulwich.repo.do_commit(
+        message=s.encode("utf-8"), commit_timestamp=commit_timestamp
+    )
+    return scm.get_rev()
 
-    rev_a = _gen("a")
-    rev_b = _gen("b")
+
+def test_list_all_commits(tmp_dir: TmpDir, scm: Git, git: Git, matcher: type[Matcher]):
+    assert git.list_all_commits() == []
+    # https://github.com/libgit2/libgit2/issues/6336
+    now = time.time()
+
+    rev_a = _gen(scm, "a", commit_timestamp=now - 10)
+    rev_b = _gen(scm, "b", commit_timestamp=now - 8)
+    rev_c = _gen(scm, "c", commit_timestamp=now - 5)
+    rev_d = _gen(scm, "d", commit_timestamp=now - 2)
+
+    assert git.list_all_commits() == [rev_d, rev_c, rev_b, rev_a]
+
+    scm.gitpython.git.reset(rev_b, hard=True)
+    assert git.list_all_commits() == [rev_b, rev_a]
+
+
+def test_list_all_commits_branch(
+    tmp_dir: TmpDir, scm: Git, git: Git, matcher: type[Matcher]
+):
+    revs = {}
+    now = time.time()
+
+    revs["1"] = _gen(scm, "a", commit_timestamp=now - 10)
+
+    scm.checkout("branch", create_new=True)
+    revs["3"] = _gen(scm, "c", commit_timestamp=now - 9)
+
+    scm.checkout("master")
+    revs["2"] = _gen(scm, "b", commit_timestamp=now - 7)
+
+    scm.checkout("branch")
+    revs["5"] = _gen(scm, "e", commit_timestamp=now - 6)
+
+    scm.checkout("master")
+    revs["4"] = _gen(scm, "d", commit_timestamp=now - 5)
+
+    scm.checkout("branch")
+    revs["6"] = _gen(scm, "f", commit_timestamp=now - 4)
+
+    scm.checkout("master")
+    revs["7"] = _gen(scm, "g", commit_timestamp=now - 3)
+    revs["8"] = scm.merge("branch", msg="merge branch")
+
+    inv_map = {v: k for k, v in revs.items()}
+    assert [inv_map[k] for k in git.list_all_commits()] == [
+        "8",
+        "7",
+        "6",
+        "4",
+        "5",
+        "2",
+        "3",
+        "1",
+    ]
+
+
+def test_list_all_tags(tmp_dir: TmpDir, scm: Git, git: Git, matcher: type[Matcher]):
+    rev_a = _gen(scm, "a")
     scm.tag("tag")
-    rev_c = _gen("c")
-    scm.gitpython.git.reset(rev_a, hard=True)
-    scm.set_ref("refs/foo/bar", rev_c)
+    rev_b = _gen(scm, "b")
+    scm.tag("annotated", annotated=True, message="Annotated Tag")
+    rev_c = _gen(scm, "c")
+    rev_d = _gen(scm, "d")
+    assert git.list_all_commits() == matcher.unordered(rev_d, rev_c, rev_b, rev_a)
 
-    assert git.list_all_commits() == matcher.unordered(rev_a, rev_b)
+    rev_e = _gen(scm, "e")
+    scm.tag(
+        "annotated2",
+        target="refs/tags/annotated",
+        annotated=True,
+        message="Annotated Tag",
+    )
+    assert git.list_all_commits() == matcher.unordered(
+        rev_e, rev_d, rev_c, rev_b, rev_a
+    )
+
+    rev_f = _gen(scm, "f")
+    scm.tag(
+        "annotated3",
+        target="refs/tags/annotated2",
+        annotated=True,
+        message="Annotated Tag 3",
+    )
+    assert git.list_all_commits() == matcher.unordered(
+        rev_f, rev_e, rev_d, rev_c, rev_b, rev_a
+    )
+
+    scm.gitpython.git.reset(rev_a, hard=True)
+    assert git.list_all_commits() == matcher.unordered(rev_b, rev_a)
+
+
+def test_list_all_commits_dangling_annotated_tag(tmp_dir: TmpDir, scm: Git, git: Git):
+    rev_a = _gen(scm, "a")
+    scm.tag("annotated", annotated=True, message="Annotated Tag")
+
+    _gen(scm, "b")
+
+    # Delete branch pointing to rev_a
+    scm.checkout(rev_a)
+    scm.gitpython.repo.delete_head("master", force=True)
+
+    assert git.list_all_commits() == [rev_a]  # Only reachable via the tag
+
+
+def test_list_all_commits_orphan(
+    tmp_dir: TmpDir, scm: Git, git: Git, matcher: type[Matcher]
+):
+    rev_a = _gen(scm, "a")
+
+    # Make an orphan branch
+    scm.gitpython.git.checkout("--orphan", "orphan-branch")
+    rev_orphan = _gen(scm, "orphanfile")
+
+    assert rev_orphan != rev_a
+    assert git.list_all_commits() == matcher.unordered(rev_orphan, rev_a)
+
+
+def test_list_all_commits_refs(
+    tmp_dir: TmpDir, scm: Git, git: Git, matcher: type[Matcher]
+):
+    assert git.list_all_commits() == []
+
+    rev_a = _gen(scm, "a")
+
+    assert git.list_all_commits() == [rev_a]
+    rev_b = _gen(scm, "b")
+    scm.set_ref("refs/remotes/origin/feature", rev_b)
+    assert git.list_all_commits() == matcher.unordered(rev_b, rev_a)
+
+    # also add refs/exps/foo/bar
+    rev_c = _gen(scm, "c")
+    scm.set_ref("refs/exps/foo/bar", rev_c)
+    assert git.list_all_commits() == matcher.unordered(rev_c, rev_b, rev_a)
+
+    # Dangling/broken ref ---
+    scm.set_ref("refs/heads/bad-ref", "deadbeefdeadbeefdeadbeefdeadbeefdeadbeef")
+    with pytest.raises(Exception):  # noqa: B017, PT011
+        git.list_all_commits()
+    scm.remove_ref("refs/heads/bad-ref")
+
+    scm.gitpython.git.reset(rev_a, hard=True)
+    assert git.list_all_commits() == matcher.unordered(rev_b, rev_a)
+
+
+def test_list_all_commits_detached_head(
+    tmp_dir: TmpDir, scm: Git, git: Git, matcher: type[Matcher]
+):
+    rev_a = _gen(scm, "a")
+    rev_b = _gen(scm, "b")
+    rev_c = _gen(scm, "c")
+    scm.checkout(rev_b)
+
+    assert scm.pygit2.repo.head_is_detached
+    assert git.list_all_commits() == matcher.unordered(rev_c, rev_b, rev_a)
 
 
 @pytest.mark.skip_git_backend("pygit2")
@@ -477,9 +642,9 @@ def test_merge(tmp_dir: TmpDir, scm: Git, git: Git, squash: bool):
 
     scm.checkout("master")
 
+    tmp_dir.gen("foo", "baz")
+    scm.add_commit("foo", message="baz")
     with pytest.raises(MergeConflictError):
-        tmp_dir.gen("foo", "baz")
-        scm.add_commit("foo", message="baz")
         git.merge(branch, commit=not squash, squash=squash, msg="merge")
 
     scm.gitpython.git.reset(init_rev, hard=True)
@@ -866,7 +1031,7 @@ async def test_git_ssh(
     git: Git,
     sftp: SFTPClient,
     ssh_connection: SSHClientConnection,
-    ssh_conn_info: Dict[str, Any],
+    ssh_conn_info: dict[str, Any],
 ):
     host, port = ssh_conn_info["host"], ssh_conn_info["port"]
     key_filename = ssh_conn_info["client_keys"]
@@ -907,6 +1072,7 @@ async def test_git_ssh(
 
 @pytest.mark.parametrize("scheme", ["", "file://"])
 @pytest.mark.parametrize("shallow_branch", [None, "master"])
+@pytest.mark.parametrize("bare", [True, False])
 def test_clone(
     tmp_dir: TmpDir,
     scm: Git,
@@ -914,16 +1080,108 @@ def test_clone(
     tmp_dir_factory: TempDirFactory,
     scheme: str,
     shallow_branch: Optional[str],
+    bare: bool,
 ):
     tmp_dir.gen("foo", "foo")
     scm.add_commit("foo", message="init")
     rev = scm.get_rev()
 
     target_dir = tmp_dir_factory.mktemp("git-clone")
-    git.clone(f"{scheme}{tmp_dir}", str(target_dir), shallow_branch=shallow_branch)
+    git.clone(
+        f"{scheme}{tmp_dir}",
+        str(target_dir),
+        shallow_branch=shallow_branch,
+        bare=bare,
+    )
     target = Git(str(target_dir))
     assert target.get_rev() == rev
-    assert (target_dir / "foo").read_text() == "foo"
+    if bare:
+        assert not (target_dir / "foo").exists()
+    else:
+        assert (target_dir / "foo").read_text() == "foo"
+        assert (target_dir / "foo").read_text() == "foo"
+    fs = target.get_fs(rev)
+    with fs.open("foo", mode="r") as fobj:
+        assert fobj.read().strip() == "foo"
+
+
+@pytest.fixture
+def proxy_server():
+    class _ProxyServer(ProxyTestCase):
+        pass
+
+    _ProxyServer.setUpClass()
+    assert _ProxyServer.PROXY
+    yield f"http://{_ProxyServer.PROXY.flags.hostname}:{_ProxyServer.PROXY.flags.port}"
+    _ProxyServer.tearDownClass()
+
+
+def test_clone_proxy_server(proxy_server: str, scm: Git, git: Git, tmp_dir: TmpDir):
+    url = "https://github.com/iterative/dvcyaml-schema"
+
+    p = (
+        Path(os.environ["HOME"] if "HOME" in os.environ else os.environ["USERPROFILE"])
+        / ".gitconfig"
+    )
+    p.write_text(BAD_PROXY_CONFIG)
+    with pytest.raises(Exception):  # noqa: PT011, B017
+        git.clone(url, "dir")
+
+    mock_config_content = f"""[http]\n
+proxy = {proxy_server}
+[https]
+proxy = {proxy_server}
+"""
+
+    p.write_text(mock_config_content)
+    git.clone(url, "dir")
+
+
+def test_iter_remote_refs_proxy_server(proxy_server: str, scm: Git, tmp_dir: TmpDir):
+    url = "https://github.com/iterative/dvcyaml-schema"
+    git = GitBackends.DEFAULT["dulwich"](".")
+
+    p = (
+        Path(os.environ["HOME"] if "HOME" in os.environ else os.environ["USERPROFILE"])
+        / ".gitconfig"
+    )
+    p.write_text(BAD_PROXY_CONFIG)
+    with pytest.raises(Exception):  # noqa: PT011, B017
+        list(git.iter_remote_refs(url))
+
+    mock_config_content = f"""[http]
+proxy = {proxy_server}
+[https]
+proxy = {proxy_server}
+"""
+
+    p.write_text(mock_config_content)
+    res = list(git.iter_remote_refs(url))
+    assert res
+
+
+@pytest.mark.skip_git_backend("gitpython")
+def test_fetch_refspecs_proxy_server(
+    proxy_server: str, scm: Git, git: Git, tmp_dir: TmpDir
+):
+    url = "https://github.com/iterative/dvcyaml-schema"
+
+    p = (
+        Path(os.environ["HOME"] if "HOME" in os.environ else os.environ["USERPROFILE"])
+        / ".gitconfig"
+    )
+    p.write_text(BAD_PROXY_CONFIG)
+    with pytest.raises(Exception):  # noqa: PT011, B017
+        git.fetch_refspecs(url, ["refs/heads/master:refs/heads/master"])
+
+    mock_config_content = f"""[http]
+proxy = {proxy_server}
+[https]
+proxy = {proxy_server}
+"""
+
+    p.write_text(mock_config_content)
+    git.fetch_refspecs(url, "refs/heads/master:refs/heads/master")
 
 
 @pytest.mark.skip_git_backend("pygit2")
@@ -932,7 +1190,7 @@ def test_fetch(tmp_dir: TmpDir, scm: Git, git: Git, tmp_dir_factory: TempDirFact
     scm.add_commit("foo", message="init")
 
     target_dir = tmp_dir_factory.mktemp("git-clone")
-    git.clone(str(tmp_dir), (target_dir))
+    git.clone(str(tmp_dir), target_dir)
     target = Git(str(target_dir))
 
     scm.add_commit("bar", message="update")
@@ -1074,7 +1332,7 @@ def test_is_dirty_untracked(
 def test_backend_func(
     tmp_dir: TmpDir,
     scm: Git,
-    backends: List[str],
+    backends: list[str],
     mocker: MockerFixture,
 ):
     from functools import partial
@@ -1085,3 +1343,61 @@ def test_backend_func(
     mock = mocker.spy(backend, "add")
     scm.add(["foo"])
     mock.assert_called_once_with(["foo"])
+
+
+def test_tag(tmp_dir: TmpDir, scm: Git, git: Git):
+    tmp_dir.gen("foo", "foo")
+    scm.add_commit("foo", message="init")
+    rev = scm.get_rev()
+
+    git.tag("lightweight")
+    assert scm.resolve_rev("lightweight") == rev
+
+    git.tag("annotated", annotated=True, message="message")
+    assert scm.resolve_rev("annotated") == rev
+
+
+def test_get_tag(tmp_dir, scm: Git, git: Git):
+    tmp_dir.gen("foo", "foo")
+    scm.add_commit("foo", message="init")
+    rev = scm.get_rev()
+
+    assert git.get_tag("nonexistent") is None
+
+    scm.tag("lightweight")
+    assert git.get_tag("lightweight") == rev
+
+    scm.tag("annotated", annotated=True, message="message")
+    tag = git.get_tag("annotated")
+    assert isinstance(tag, GitTag)
+    assert tag.target == rev
+    assert tag.message.strip() == "message"
+
+
+@pytest.mark.skip_git_backend("gitpython")
+def test_config(tmp_dir, scm: Git, git: Git):
+    tmp_dir.gen({".git": {"config": "[test]\nfoo = true\n"}})
+    tmp_dir.gen(".otherconfig", "[test]\nfoo = false\n")
+    config = git.get_config()
+    assert config.get(("test",), "foo") == "true"
+    assert config.get_bool(("test",), "foo") is True
+
+    config = git.get_config(".otherconfig")
+    assert config.get(("test",), "foo") == "false"
+    assert config.get_bool(("test",), "foo") is False
+
+
+@pytest.mark.skip_git_backend("dulwich")
+def test_check_attr(tmp_dir, scm: Git, git: Git):
+    tmp_dir.gen("foo.txt", "foo")
+    tmp_dir.gen(".gitattributes", "*.txt text")
+    scm.add_commit([".gitattributes", "foo"], message="init")
+    rev = scm.get_rev()
+    assert git.check_attr("foo.txt", "text") is True
+    assert git.check_attr("foo.txt", "filter") is None
+
+    tmp_dir.gen(".gitattributes", "*.txt -text filter=lfs")
+    assert git.check_attr("foo.txt", "text") is False
+    assert git.check_attr("foo.txt", "filter") == "lfs"
+    assert git.check_attr("foo.txt", "text", source=rev) is True
+    assert git.check_attr("foo.txt", "filter", source=rev) is None
